-- Max OOF Utility
-- by Robert 'SuperSheep' Piontek

global myImage  = "selfloat.bmp"
global myImageA = "selfloat_a.bmp"
global myLogo   = "maxooflogo.bmp"
global curPath  = "C:\\"
global wildSearch = true
global bwcolor = 0
global props    = #("None", \
                    "Facing", \
                    "Rotate", \
                    "Turret/FOV", \
                    "Glow/Thruster", \
                    "Custom(Logo)", \
                    "Shell", \
                    "FrontFace", \
                    "Jitter", \
                    "Viewer", \
                    "Monitor(01-08)", \
                    "Layer")
global monitors = #("Monitor01 - Left HUD Display", \
                    "Monitor02 - Ship Strength", \
                    "Monitor03 - Right HUD Display", \
                    "Monitor04 - Left Energy Indicator", \
                    "Monitor05 - Right Energy Indicator", \
                    "Monitor06 - Shields", \
                    "Monitor07 - Afterburn Indicator", \
                    "Monitor08 - Unknown")
global curPropShown = 1
global winSize = [250,440]
global mtlSlot = 1
-- Property initial/save states
struct s_fov  (fov,spr,rt)
struct s_glow (color,radius)
global prop_rotate  = 1.0
global prop_monitor = 1
global prop_fov     = s_fov  fov:90.0 spr:1.0 rt:0.0
global prop_glow    = s_glow color:blue radius:1.0

-- Takes a string with single \'s and returns one with \\'s
fn doubleSlash str = (for i = str.count to 1 by -1 do (if str[i] == "\\" then str = replace str i 1 "\\\\"); return str)
-- Takes a string with \\'s and returns one with \'s
fn singleSlash str = (for i = str.count-1 to 1 by -1 do (if str[i] == "\\" and str[i+1] == "\\" then str = replace str i 1 ""); return str)
-- Removes file extension from str
fn removeExt str = (for i = str.count-1 to 1 by -1 do (if str[i] == "." then (str = replace str i 4 ""; exit));	return str)

-- Rename diffuse texture to long name
fn renameDiffuse mat =
(
	dif_map = mat.diffuseMap

	-- Have we already got a fully qualified path name and file exists?
	f = getFilenamePath(dif_map.filename)
	if matchPattern f pattern:"*\\*" then (f = getFiles(dif_map.filename); if f.count == 1 then return 0)

	-- Store as much of the file name as we can
	search_name = curPath + mat.name

	-- Can we find this name?
	if mat.name.count<17 then
	(
		f = getFiles(search_name + ".tga")
		if f.count == 1 then (dif_map.name = mat.name; dif_map.filename = singleSlash(f[1]); return 0)
	)

	-- Can we find the name with a wildcard search?
	if wildSearch == true then
	(
		f = getFiles(search_name + "*.tga")
		if f.count == 1 then
		(
			-- We found it, so exit
			mat.name         = removeExt(filenameFromPath(f[1]))
			dif_map.name     = mat.name
			dif_map.filename = singleSlash(f[1])
			return 0
		)
	)

	-- We now need to prompt user to find texture
	f = getOpenFileName filename:(search_name + ".tga") caption:("Missing texture: " + mat.name) types:"TGA Files(*.TGA)|*.tga|All files(*.*)|*.*|"
	if f != undefined then
	(
		-- Store new map and material name and update Current Path
		mat.name         = removeExt(filenameFromPath(f))
		dif_map.name     = mat.name
		dif_map.filename = singleSlash(f)
		curPath          = getFileNamePath(f)
	)
)

-- Renames all bitmaps on materials
fn renameMaterials =
(
	if ClassOf meditMaterials[mtlSlot] == Standard then
		if ClassOf meditMaterials[mtlSlot].diffuseMap == BitmapTexture then
			renameDiffuse meditMaterials[mtlSlot]
	if ClassOf meditMaterials[mtlSlot] == MultiMaterial then
	(
		for i = 1 to meditMaterials[mtlSlot].material.count do
		(
			if ClassOf meditMaterials[mtlSlot].material[i] == Standard then
				if ClassOf meditMaterials[mtlSlot].material[i].diffuseMap == BitmapTexture then
					renameDiffuse meditMaterials[mtlSlot].material[i]
		)
	)
)

-- Create Material Editor materials from objects in scene
fn createMaterialsFromObjects =
(
	if selection.count == 0 then (messageBox "No objects selected!"; return 0)
	curMatId = 0
	matnames = #()
	materials = #()
	-- Create a material list from objects
	for i = 1 to selection.count do
	(
		o = selection[i]
		if ClassOf o == Editable_Mesh then
		(
			mat = o.material
			if mat == undefined then continue
			if ClassOf mat == Standard then
			(
				if ClassOf mat.diffuseMap != BitmapTexture then (o.material = undefined; continue)
			)
			for curFace = 1 to o.numFaces do
			(
				matId = getFaceMatId o curFace
				local submat = mat
				if (ClassOf mat==MultiMaterial) and (matId>0) and (matid<=mat.material.count) then submat = mat.material[matId]
				local index = finditem matnames submat.name
				if index == 0 then
				(
					append matnames submat.name
					append materials submat
					curMatId += 1
					setFaceMatId o curFace curMatId
				)
				else setFaceMatId o curFace index
			)
		)
		update o
	)
	-- Create materials
	if curMatId == 1 then
	(
		setMeditMaterial (mtlSlot) materials[1]
		-- Show texture in viewport
		if ClassOf meditMaterials[mtlSlot].diffuseMap == BitmapTexture then
			showTextureMap meditMaterials[mtlSlot] meditMaterials[mtlSlot].diffuseMap on
	)
	else if curMatId > 1 then
	(
		mat = multiMaterial numsubs:curMatId name:("Material #" + (mtlSlot as string))
		mat.materialList = materials
		mat.names = matnames
		if mat != undefined then
		(
			setMeditMaterial (mtlSlot) mat
			-- Show texture in viewport
			for i = 1 to curMatId do
			(
				if ClassOf meditMaterials[mtlSlot].material[i].diffuseMap == BitmapTexture then
					showTextureMap meditMaterials[mtlSlot].material[i] meditMaterials[mtlSlot].material[i].diffuseMap on
			)
		)
	)
	-- Rename the materials
	renameMaterials()
)

-- Assign material to selected objects
fn assignMaterials =
(
	-- Assign material to all objects
	for i = 1 to selection.count do
	(
		o = selection[i]
		if ClassOf o == Editable_Mesh and o.material != undefined then
		(
			o.material = undefined
			update o
			if ((findString o.name "$attach")==1) or ((findString o.name "$normal")==1) or \
			   ((findString o.name "$gun")   ==1) or ((findString o.name "$ground")==1) or \
			   ((findString o.name "$dummy")==1)  or ((findString (GetUserPropBuffer o) "$glow")==1) then continue
			o.material = meditMaterials[mtlSlot];
			update o
		)
	)
	redrawViews()
)

-- Welds all redundant vertices
fn weldAllVerts o=
(
	j = o.numVerts
	k = j
	tmppos = o.verts[j].pos
	do
	(
		if o.verts[j].pos!=tmppos then
		(
			if k>(j+1) then (setVertSelection o #{(j+1)..(k)}; meshOps.weld o; deselect o.selectedVerts)
			tmppos = o.verts[j].pos
			k = j
		)
	) while (j-=1) > 0
	-- Do last set of vertices
	if k>(j+1) then (setVertSelection o #{1..(k)}; meshOps.weld o; deselect o.selectedVerts)
)

-- Colorize Model
fn colorize style =
(
	for i = 1 to objects.count do
	(
		o = objects[i]
		if ClassOf o == Editable_Mesh then
		(
			if      (findString o.name "$attach") == 1 then o.wirecolor = orange
			else if (findString o.name "$normal") == 1 then o.wirecolor = color 255 0 255
			else if	(findString o.name "$gun")    == 1 then o.wirecolor = red
			else if (findString o.name "$ground") == 1 then o.wirecolor = green
			else if (findString o.name "$dummy")  == 1 then o.wirecolor = yellow
			else if ((findString (GetUserPropBuffer o) "$glow")==1) then continue
			else if (style == 2) then
			(
				vpcolor = getVPortBGColor()
				vpgray  = (vpcolor.r + vpcolor.g + vpcolor.b)/3
				if (bwcolor += 16) > 255 then bwcolor = 0
				if (bwcolor > vpgray-25) and (bwcolor < vpgray+25) then bwcolor = vpgray+25
				if bwcolor>255 then bwcolor = 255
				o.wireColor = color bwcolor bwcolor bwcolor
			)
			else
			(
				vpcolor = getVPortBGColor()
				mycolor = color (random 0 196) (random 0 196) (random 0 196)
				if (mycolor.r>vpcolor.r-25) and (mycolor.r<vpcolor.r+25) then mycolor.r = vpcolor.r-25
				if (mycolor.g>vpcolor.g-25) and (mycolor.g<vpcolor.g+25) then mycolor.g = vpcolor.g-25
				if (mycolor.b>vpcolor.b-25) and (mycolor.b<vpcolor.b+25) then mycolor.b = vpcolor.b-25
				if mycolor.r<0 then mycolor.r=0
				if mycolor.g<0 then mycolor.g=0
				if mycolor.b<0 then mycolor.b=0
				o.wirecolor = mycolor
			)
		)
		update o
	)
)

-- Set Custom property
fn SetCustomProperty type =
(
	numProps = 0
	prop = "None"
	if selection.count == 0 then return 0
	for i = 1 to selection.count do
	(
		o = selection[i]
		if ClassOf o == Editable_Mesh then
		(
			sfov  = stringStream ""
			sglow = stringStream ""
			format "%, %, %" prop_fov.fov prop_fov.spr prop_fov.rt to:sfov
			format "%, %, %, %" (prop_glow.color.red/255) (prop_glow.color.green/255) (prop_glow.color.blue/255) prop_glow.radius to:sglow
			SetUserPropBuffer o ""
			if ((findString o.name "$attach")==1) or ((findString o.name "$normal")==1) or \
			   ((findString o.name "$gun")   ==1) or ((findString o.name "$ground")==1) or \
			   ((findString o.name "$dummy")==1) then continue
			case type of
			(
				2:  SetUserPropBuffer o "$facing"
				3:  SetUserProp       o "$rotate" prop_rotate
				4:  SetUserProp       o "$fov"    (sfov as string)
				5: (SetUserProp       o "$glow"   (sglow as string); o.wirecolor = prop_glow.color)
				6:  SetUserPropBuffer o "$custom"
				7:  SetUserPropBuffer o "$shell"
				8:  SetUserPropBuffer o "$frontface"
				9:  SetUserPropBuffer o "$jitter"
				10: SetUserPropBuffer o "$viewer"
				11: SetUserPropBuffer o ("$monitor0" + prop_monitor as string)
				12: SetUserPropBuffer o "$layer"
			)
			prop = GetUserPropBuffer o
			if prop == "" then prop = "None"
			numProps += 1
		)
	)
	if numProps>0 then messageBox ("Property "+prop+" set for "+ numProps as string + " object(s)")
)


-- ******************** Floating Rollout window ********************
--

-- ********** Sub-Object(s) Property rollouts **********
--
rollout Prop1 "None"   (label lab01 "Sub-Object(s) has No properties."   align:#left)
rollout Prop2 "Facing" (label lab02 "Sub-Object(s) always Faces player." align:#left)
rollout Prop3 "Rotate"
(
	label   lab03     "Sub-Object(s) is a Rotator." align:#left
	spinner spnrotate "Rotates once every: "        align:#left across:2 fieldWidth:40 range:[0.1,999,prop_rotate]
	label   lab04     " seconds"                    align:#right
	on spnrotate changed val do (prop_rotate = val)
)
rollout Prop4 "Turret/FOV"
(
	label   lab05    "Sub-Object(s) is a Turret." align:#left
	spinner spnfov   "Field of View: "            align:#right offset:[-30,2]  fieldWidth:40 range:[0.0,360.0,prop_fov.fov]
	spinner spnspr   "Seconds per Rotation: "     align:#right offset:[-30,-3] fieldWidth:40 range:[0.1,60.0,prop_fov.spr]
	spinner spnrt    "Reaction Time: "            align:#right offset:[-30,-3] fieldWidth:40 range:[0.0,10.0,prop_fov.rt]
	on spnfov changed val do (prop_fov.fov = val)
	on spnspr changed val do (prop_fov.spr = val)
	on spnrt  changed val do (prop_fov.rt  = val)
)
rollout Prop5 "Glow/Thruster"
(
	label       lab06         "Sub-Object(s) is a Glow/Thruster."  align:#left
	colorpicker cpglowcolor   "Glow Color: "                       align:#left  across:2 color:prop_glow.color
	spinner     spnglowradius "Radius: "                           align:#right offset:[0,4] range:[0.0,999.9,prop_glow.radius]
	on cpglowcolor   changed val do (prop_glow.color  = val)
	on spnglowradius changed val do (prop_glow.radius = val)
)
rollout Prop6  "Custom"    (label lab07 "Sub-Object(s) has Custom texture/color." align:#left)
rollout Prop7  "Shell"     (label lab08 "Sub-Object(s) is a Door Shell."          align:#left)
rollout Prop8  "FrontFace" (label lab09 "Sub-Object(s) is a Door Front."          align:#left)
rollout Prop9  "Jitter"    (label lab10 "Sub-Object(s) Jitters."                  align:#left)
rollout Prop10 "Viewer"    (label lab11 "Sub-Object(s) is Viewer."                align:#left)
rollout Prop11 "Monitor(01-08)"
(
	label        lab12 "Sub-Object(s) is a Monitor." align:#left
	dropdownlist dlMonitor                           align:#left width:(winSize.x-54) items:monitors selection:prop_monitor
	on dlMonitor selected sel do (prop_monitor = sel)
)
rollout Prop12 "Layer" (label lab13 "Sub-Object(s) is drawn after model." align:#left)


-- ********** Main Rollout(s) **********
--
-- About rollout
rollout main "About"
(
	label  label01 "Max OOF Utility v1.0"
	bitmap logo width:132 height:68 fileName:myLogo
	label  label02 "by Rob 'SuperSheep' Piontek"
	label  label03 "See Readme.txt for instructions"
)
-- Object rollout
rollout model "Model"
(
	group "Mesh"
	(
		button       btnWeldVerts  "Weld Redundant Verts" align:#left  width:(winSize.x-54)
		progressbar  pbProgress                           align:#left  width:(winSize.x-54) height:8 value:0 color:red
		button       btnColorize   "Colorize"             align:#left  width:((winSize.x-59)/2) across:2
		button       btnColorizeBW "B/W Colorize"         align:#right width:((winSize.x-59)/2)
	)
	group "Materials"
	(
		button   btnSelFloater "+"                       align:#left  width:20 height:20 across:3 toolTip:"Select object(s)" images:#(myImage,myImageA,2,1,1,2,2)
		button   btnCreateMat "Create Materials"         align:#left  pos:[(btnSelFloater.pos.x+30),(btnSelFloater.pos.y)] width:((winSize.x-70)*.5)
		spinner  spnMatSlot   "Slot: "                   align:#right fieldWidth:30 range:[1,24,(mtlSlot)] type:#integer offset:[0,4]
		checkBox cbWildSearch "Wildcard Material Search" align:#left  checked:wildSearch
	)
	-- Fix 3DS file button pressed
	on btnWeldVerts pressed do
	(
		setCommandPanelTaskMode mode:#modify
		try
		(
			disableSceneRedraw()
			undo off
			(
				for i = 1 to objects.count do
				(
					pbProgress.value = 100*i/objects.count
					obj = objects[i]
					if ClassOf obj == Editable_Mesh then
					(
						modPanel.setCurrentObject obj
						subObjectLevel = 1
						weldAllVerts(obj)
						subObjectLevel = 0
						update obj
						clearSelection()
					)
				)
			)
			enableSceneRedraw()
			pbProgress.value = 0
		)
		catch
		(
			enableSceneRedraw()
			pbProgress.value = 0
			throw()
		)
		redrawViews()
	)
	on btnColorize   pressed          do (colorize 1)
	on btnColorizeBW pressed          do (colorize 2)
	on btnSelFloater pressed          do (max selection floater)
	on btnCreateMat  pressed          do (createMaterialsFromObjects(); assignMaterials())
	on spnMatSlot    changed val      do (mtlSlot = val)
	on cbWildcard    changed theState do (wildSearch = theState)
)

-- Sub-Object rollout
rollout submodel "Sub Model"
(
	button       btnSelFloater "+"             align:#left  width:20 height:20 across:3 toolTip:"Select object(s)" images:#(myImage,myImageA,2,1,1,2,2)
	button       btnSetProp    "Set Property:" align:#left  pos:[(btnSelFloater.pos.x+30),(btnSelFloater.pos.y)] width:((winSize.x-100)*.5)
	dropdownlist dlProperty                    align:#right width:((winSize.x-75)*.5) height:13 items:props selection:1

	on dlProperty selected sel do
	(
		if sel!=curPropShown then
		(
			cp = (curPropShown as string)
			execute("removerollout Prop"+cp+" MaxOOFUtility")
			execute("addrollout Prop"+(sel as string)+" MaxOOFUtility")
			curPropShown = sel
		)
	)
	on btnSetProp    pressed do (SetCustomProperty curPropShown)
	on btnSelFloater pressed do (max selection floater)
)

-- Create Floating Rollout
file = getFiles(myImage);  if file.count == 1 then myImage  = file[1]
file = getFiles(myImageA); if file.count == 1 then myImageA = file[1]
file = getFiles(myLogo);   if file.count == 1 then myLogo   = file[1]
MaxOOFUtility = newrolloutfloater "MaxOOFUtility" (winSize.x) (winSize.y)
addrollout main     MaxOOFUtility
addrollout model    MaxOOFUtility
addrollout submodel MaxOOFUtility
addrollout Prop1    MaxOOFUtility